tree grammar MathTree;

options { 
  ASTLabelType = CommonTree;
  tokenVocab = MathParser; // because it contains a token spec.
  output = template;
}

@header {
  package com.ociweb.math;
}

assign
  : ^(ASSIGN NAME value)
    -> assign(name={$NAME}, value={$value.st})
  ;
  
combine
  : ^(COMBINE into=NAME op=SIGN lhs=NAME rhs=NAME)
    -> combine(into={$into}, lhs={$lhs}, op={$op}, rhs={$rhs})
  ;
  
// This demonstrates using a validating semantic predicate
// to prevent functions from being redefined.
define
  : ^(DEFINE fn=NAME variable=NAME polynomial)
    -> define(name={$fn},
              variable={$variable},
              polynomial={$polynomial.st})
  ;
  
derivative: ^(DERIVATIVE NAME) -> derivative(name={$NAME});
  
functionEval returns [double result]
  : ^(FUNCTION fn=NAME v=NUMBER)
    -> functionEvalForValue(name={$fn}, value={$v})
  | ^(FUNCTION fn=NAME v=NAME)
    -> functionEvalForVariable(name={$fn}, var={$v})
  ;
  
help: HELP -> help();
  
interactiveStatement
  : help -> {$help.st}
  | list -> {$list.st}
  ;

list
  : ^(LIST FUNCTIONS) -> listFunctions()
  | ^(LIST VARIABLES) -> listVariables()
  ;

polynomial
  : ^(POLYNOMIAL terms+=term[""] (s=SIGN terms+=term[$s.text])*)
    -> polynomial(terms={$terms})
  ;

print
  : ^(PRINT (targets+=printTarget)*) -> print(targets={$targets});

printTarget
  : v=NUMBER -> text(value={$v})
  | v=STRING_LITERAL -> text(value={$v})
  | NAME -> variable(name={$NAME})
  | ^(FUNCTION NAME) -> function(name={$NAME})
  | functionEval -> {$functionEval.st}
  | derivative -> {$derivative.st}
  ;
 
script: statements+=statement* -> script(statements={$statements});

statement
  : assign -> {$assign.st}
  | combine -> {$combine.st}
  | define -> {$define.st}
  | interactiveStatement -> {$interactiveStatement.st}
  | print -> {$print.st}
  ;

term[String sign]
  : ^(TERM coef=NUMBER)
    {
      Double c = Double.parseDouble($coef.text);
      if ("-".equals(sign)) c = -c;
    }
    -> term(coef={c})
  | ^(TERM coef=NUMBER? var=NAME exp=NUMBER?)
    {
      Double c = Double.parseDouble($coef.text);
      if ("-".equals(sign)) c = -c;
    }
    -> term(coef={c}, var={$var}, exp={$exp})
  ;

value
  : NUMBER -> number(text={$NUMBER})
  | NAME -> variable(name={$NAME})
  | functionEval -> {$functionEval.st}
  ;
